import AutoRPE.UtilsRPE.BasicFunctions as BasicFunctions
import AutoRPE.UtilsRPE.MaskCreator as MaskCreator
import AutoRPE.UtilsRPE.RegexPattern as RegexPattern
import AutoRPE.UtilsRPE.VariablePrecision as VariablePrecision
import AutoRPE.UtilsRPE.SourceManager as SourceManager

import re


def is_function_declaration_statement(line):
    # Remove matches inside write(*,*) statements
    # Check for function declaration syntax
    is_start = re.search(RegexPattern.function_declaration, line, re.MULTILINE | re.IGNORECASE)
    if is_start:
        return True
    return False


def is_end_function_statement(line):
    is_end = re.match(RegexPattern.end_function_declaration, line, re.I)
    if is_end:
        return True
    return False


def is_a_number(string):
    """Check if a string is a number and returns the type"""

    # Check if is just white spaces
    if not string.strip():
        return False
    else:
        string = string.lower().strip()

    # Remove unary operator if present
    string = BasicFunctions.remove_unary_operator(string)

    if has_intrinsics(string):
        return False

    # Checks for number in exponential notation
    exp = re.match(RegexPattern.number_exp_DPE_notation, string)
    # Checks for number with precision specified
    real_with_precision = re.match("^" + RegexPattern.real_number_with_precision + "$", string)
    if real_with_precision:
        # In case the number is a hardcoded real with a type specification,
        # check the lookup table to know to which type it corresponds and return it
        new_type = VariablePrecision.lookup_table[real_with_precision.group(2)]
        return new_type

    if exp:
        return VariablePrecision.real_id['wp']

    integer_with_precision = re.match(RegexPattern.integer_number_with_precision, string)
    if integer_with_precision:
        return "integer"
    # Check for pure integer and real numbers
    try:
        float(string)
        if string.isdigit():
            return "integer"
        else:
            return VariablePrecision.real_id['wp']
    except ValueError:
        return False


def is_hardcoded_array(string, array=None):
    """Check if passed string is in form (/a, b/). If the array parameter is passed, fill it with [a,b]"""
    import AutoRPE.UtilsRPE.CallManager as CallManager
    match = re.match(RegexPattern.hardcoded_array, string)
    if match:
        if array is not None:
            # remove_outer_brackets will remove parenthesis just if present
            content = "(" + BasicFunctions.remove_outer_brackets(match.group(2)) + ")"
            array.extend(CallManager.find_call_arguments(content))
        return True
    return False


def is_pointer_assignment(string):
    pattern = ".* => .*"
    if re.search(pattern, string, flags=re.I):
        return True
    return False


def is_logical_comparison(string):
    for element in SourceManager.comparison_operators:
        if string.lower().count(element):
            return True
    return False


def is_keyword_argument(argument, key_arg=None):
    if re.search(RegexPattern.key_word_argument, argument):
        if key_arg is not None:
            [key, argument] = argument.split("=", 1)
            key = key.strip()
            argument = argument.strip()
            key_arg.extend([key, argument])
        return True
    return False


def is_array(contents, check_sliced=False):
    import AutoRPE.UtilsRPE.CallManager as CallManager
    """
        This function first does a first filter checking that the syntax is similar to a function call and afterwards
        uses one of the two possible approaches, depending on having a vault available or not.
    """
    # The syntax is be the same for vector and function (unless vector is sliced)
    m = re.match(RegexPattern.call_to_function, contents.strip(), re.I)
    if not m:
        return False

    if contents == BasicFunctions.close_brackets(contents):
        arguments = CallManager.find_call_arguments(contents)
        # No arguments: it is a function or a single variable
        if arguments:
            if not check_sliced:
                return True
            else:
                # check if an argument is a slice
                for arg in arguments:
                    if re.match(r"\w*:\w*", arg) or arg == ":":
                        return True
                return False
    return False


def has_operations(string, operand=None):
    operations = ["+", "-", "*", "/"]
    # Exclude key arguments
    if is_keyword_argument(string):
        return False
    # Check if string is just white spaces
    if not string.strip():
        return False
    else:
        string = string.replace(" ", "")

    if is_array(string):
        return False

    if operand is not None:
        # If we want to split the argument by operand, store the masked element in a dictionary
        dictionary_of_replacements = {}
        string = MaskCreator.mask_string_for_check_operation(string, dictionary_of_replacements)
    else:
        string = MaskCreator.mask_string_for_check_operation(string)

    # Now that all offenders have been masked, split the elements
    for op in operations:
        # Substitute all operators with a placeholder
        string = string.replace(op, "#")
    # The string does not contains operations
    if not string.count("#"):
        return False
    else:
        # Fill the passed vector
        if operand is not None:
            # A  void unitary operators to be considered operators ( rn_Dt * (-10_wp)) => rn_Dt * -10_wp )
            string = re.sub(r"#\s*#", r"#", string)
            # Remove unary operators at the beginning of the string
            string = re.sub(r"^\s*#", "", string)

            # Split by placeholder
            arguments = string.split("#")
            # Strip from spaces due to substitutions
            arguments = [a.strip() for a in arguments]
            operand.extend(MaskCreator.unmask_from_dict(arguments, dictionary_of_replacements))
            # Substitute back all entities that were masked
        return True


def has_intrinsics(string):
    for key in SourceManager.list_of_key:
        if string.lower().count(key):
            key_match = r"\s*[ ,(+-]%s *= *\w" % key
            string = re.sub(key_match, "", string, re.I)

    # Split parenthesis opening
    if not string.count("("):
        return False

    possible_intrinsics = re.findall(r'\s*\b\w+\b\s*\(', string, re.I)

    for name in possible_intrinsics:
        # Remove last parenthesis and leading spaces
        name = name[:-1].strip()
        try:
            SourceManager.list_of_intrinsics[name.lower()]
            return name
        except KeyError:
            continue
    return None

def is_struct_element(contents: str) -> bool:
    if contents.count("%"):
        return True
    return False
    ...